#! /bin/sh
#
#   startup.sh -- スタートアップスクリプト (日本語化コンソール用, floppy版)
#
#	0.0: Jun. 19, 2004 by Dai ISHIJIMA
#

export PATH=/usr/bin:/bin:/sbin:/wipeout:/:.

# 起動時のシステムメッセージを保存
dmesg > /var/run/dmesg.boot

# ツール起動
cd /wipeout
/bin/sh mainmenu.sh

# とりあえず対話シェルを動かしておく :-)
/bin/sh -i

exit 0

# EOF
