#! /bin/sh
#
#   verify.sh -- データが消えているか確認する
#
#	使い方: /bin/sh verify.sh step ディスク
#		step: スキップする長さ
#		ディスク: デバイスファイルで指定、先頭の「/dev/r」は省く
#		例: /bin/sh verify.sh 90 ad0
#
#

step="${1:-1}"
disk="${2:-ad0}"

if [ ! -r /dev/r$disk ]; then
	echo "$0: disk '$disk' does not exist or is not readable"
	echo "Usage: $0 [step] disk"
	echo "Example: $0 -1 ad0"
	exit 1
fi

title="${TITLE:-ハードディスク消去ツール『wipe-out』}"

dmesg=/var/run/dmesg.boot
dname=`sed -n -E "/^${disk}:.*<.*>/p" $dmesg | sed 's/.*\(<.*>\).*/\1/'`
dbytes=`diskinfo -p /dev/r$disk`


# 確認のパラメータ
bytes=`diskinfo -b /dev/r$disk`
# 10Mバイトずつ確認
ibs=`expr 1024 '*' 1024`
icnt=10
# ブロックサイズ単位でのディスクサイズ
max=`expr '(' $bytes + $ibs - 1 ')' / $ibs`

# 結果
zero=0
one=0
maybe=0
unerased=0
checked=0

startsec=`tinydate '+%s'`
rems=''

skip=0
while [ $skip -lt $max ]; do
	# 計時
	cursec=`tinydate '+%s'`
	# バーグラフ
	p=`expr 100 '*' $skip / $max`
	m=`expr 1000 '*' $skip / $max`
	q=`expr $p / 2`
	i=0
	bar=''
	while [ $i -lt $q ]; do
		bar="$bar"'#'
		i=`expr $i + 1`
	done
	while [ $i -lt 50 ]; do
		bar="$bar"'_'
		i=`expr $i + 1`
	done
	# 残り時間
	elapsed=`expr $cursec - $startsec`
	if [ $elapsed -gt 10 ]; then
		remain=`expr '(' 1000 - $m ')' '*' $elapsed / $m + 1`
		if [ $remain -gt 120 ]; then
			rems=`expr '(' $remain + 59 ')' / 60`
			rems="残り およそ${rems}分"
		else
			rems="残り およそ${remain}秒"
		fi
	fi
	if [ $elapsed -gt 120 ]; then
		elas=`expr '(' $elapsed + 30 ')' / 60`
		elas="${elas}分"
	else
		elas="${elapsed}秒"
	fi
	#
	text="ディスク ${disk} のデータをチェックしています。\n"
	text="${text}${max}ブロックのうち${skip}ブロック／${p}% 終了\n"
	text="${text}${elas} 経過  ${rems}  \n\n"
	text="${text}        0%|${bar}|100%"
	text="${text}\n${errormsg}"
	dialog --title "$title" --infobox "$text" 8 72
	echo -n "$status2"
	case x$DEBUG in
		x[Yy]*)
			status2='now in debug mode...'
			errormsg=''
			sleep 1
			;;
		*)
			status=`(dd if=/dev/r$disk ibs=$ibs skip=$skip count=$icnt | check -s) 2>&1`
			case $? in
				0) zero=`expr $zero + $icnt` ;;
				1) one=`expr $one + $icnt` ;;
				2) maybe=`expr $maybe + $icnt` ;;
				*) unerased=`expr $unerased + $icnt` ;;
			esac
			status2=`echo "$status" | sed -n '/transfer/p'`
			errormsg=`echo "$status" | sed -n -E '/records (in|out)$/!p' | sed -n '/transferred in/!p'`
			;;
	esac
	# 次ループ
	checked=`expr $checked + $icnt`
	skip=`expr $skip + $icnt + $step`
done

sleep 2

cursec=`tinydate '+%s'`
elapsed=`expr $cursec - $startsec`
if [ $elapsed -gt 120 ]; then
	min=`expr $elapsed / 60`
	sec=`expr $elapsed - $min '*' 60`
	elas="${min}分${sec}秒"
else
	elas="${elapsed}秒"
fi

unerased=`expr $unerased + $maybe`
zp=`expr $zero '*' 100 / $checked`
op=`expr $one '*' 100 / $checked`
np=`expr $unerased '*' 100 / $checked`

msg="ディスク ($disk, $dbytes バイト) のチェックが終了しました (${elas})\n"
msg="${msg}全 $max ブロック中の $checked ブロックを検査しました\n"
msg="${msg}  '0' で消去されているブロック数: $zero ($zp %)\n"
msg="${msg}  '1' で消去されているブロック数: $one ($op %)\n"
msg="${msg}  それ以外 (未消去) のブロック数: $unerased ($np %)\n"

dialog	--title "$title" \
	--msgbox "$msg" 10 72

exit 0

# EOF
