#! /bin/sh
#
#   dump.sh -- ディスクのデータを表示する
#
#	使い方: /bin/sh dump.sh skip ディスク
#		skip: スキップする長さ
#		ディスク: デバイスファイルで指定、先頭の「/dev/r」は省く
#		例: /bin/sh dump.sh 0 ad0
#
#

skip="${1:-0}"
disk="${2:-ad0}"

title="■ $disk の セクタ $skip の内容"
(dd if=/dev/r$disk skip=$skip count=1 | tinydump > /tmp/dump.txt) 2>/dev/null

dialog	--title "$title" \
	--textbox /tmp/dump.txt 22 76
