#! /bin/sh
#
#   startup.sh -- スタートアップスクリプト (日本語化コンソール用)
#
#	0.0: Jun. 15, 2004 by Dai ISHIJIMA
#	0.1: Jun. 19, 2005 (for PXEBOOT)
#

PATH=/bin:/sbin:/:.
export PATH

echo -n '起動時のシステムメッセージを保存しています...'
dmesg > /var/run/dmesg.boot
echo ''

# dialogコマンドがあらかじめ存在するか? (PXEブートか?)
CDROM=PXEBOOT
if [ ! -x /usr.bin/dialog ]; then
	# 存在しない場合はCD-ROMをマウント
	if [ ! -x /usr/bin/dialog ]; then
		echo -n 'CD-ROMをマウントしています...'
		# まず、ATAPI CD-ROMをマウントしてみる
		mount_cd9660 /dev/acd0c /cdrom 1>/dev/null 2>&1
		CDROM=/dev/acd0c
		if [ ! -x /cdrom/usr.bin/dialog ]; then
			# SCSI CD-ROMをマウントしてみる
			mount_cd9660 /dev/cd0c /cdrom 1>/dev/null 2>&1
			CDROM=/dev/cd0c
		fi
		echo ''
		if [ ! -x /cdrom/usr.bin/dialog ]; then
			CDROM=NONE
			echo 'CD-ROMのマウントに失敗しました。'
			exec /bin/sh -i
		fi
	fi
	export CDROM
	# CD-ROMに入っている static link なコマンドをコピー
	mkdir /usr.bin
	cp /cdrom/usr.bin/* /usr.bin
	# umountもコピー
	rm -f /sbin/umount
	cp /cdrom/sbin/umount /sbin
fi

echo -n 'ハードディスク消去ツールの起動準備をしています...'
if [ -r /usr/lib/libc.so ]; then
	# シェアードライブラリがあれば (CDのマウントに成功していれば) 
	# ライブラリの初期化
	ldconfig -elf /usr/lib
fi

if [ -x /bin/hostname ]; then
	hostname wipeout
fi

# 一度CD-ROMをアンマウント
case $CDROM in
	/dev/*)
		umount /cdrom
		;;
esac

PATH=/wipeout:/usr.bin:/usr/bin:/usr/sbin:/bin:/sbin:/:.
export PATH
export TERMCAP=/etc/termcap
export TERM=kons25jx
echo ''

cd /wipeout
/bin/sh mainmenu.sh

# とりあえず対話シェルを動かしておく :-)
/bin/sh -i

exit 0

# EOF
